@@ -1,11 +1,12 @@
+from __future__ import print_function, division
+
 import os
 from distutils import version
 import numpy as np
 
-from exceptions import TableException
-from helpers import smart_dtype, smart_mask
-
-import atpy
+from .exceptions import TableException
+from .helpers import smart_dtype, smart_mask
+from .decorators import auto_download_to_file, auto_fileobj_to_file
 
 
 pyfits_minimum_version = version.LooseVersion('2.1')
@@ -55,7 +56,10 @@ def _list_tables(filename):
     return tables
 
 
-def read(self, filename, hdu=None, verbose=True):
+# PyFITS can handle compression, so no decompression detection
+@auto_download_to_file
+@auto_fileobj_to_file
+def read(self, filename, hdu=None, memmap=False, verbose=True):
     '''
     Read a table from a FITS file
 
@@ -69,6 +73,9 @@ def read(self, filename, hdu=None, verbose=True):
         *hdu*: [ integer ]
             The HDU to read from the FITS file (this is only required
             if there are more than one table in the FITS file)
+
+        *memmap*: [ bool ]
+            Whether PyFITS should use memory mapping
     '''
 
     _check_pyfits_installed()
@@ -85,7 +92,7 @@ def read(self, filename, hdu=None, verbose=True):
         else:
             raise TableException(tables, 'hdu')
 
-    hdulist = pyfits.open(filename)
+    hdulist = pyfits.open(filename, memmap=memmap)
     hdu = hdulist[hdu]
 
     table = hdu.data
@@ -154,7 +161,7 @@ def read(self, filename, hdu=None, verbose=True):
         format, bzero = hdu.columns[i].format[-1], hdu.columns[i].bzero
 
         if bzero and format in ['B', 'I', 'J']:
-            data = pyfits.rec.recarray.field(hdu.data, i)
+            data = np.rec.recarray.field(hdu.data, i)
             if format == 'B' and bzero == -128:
                 data = (data.astype(np.int16) + bzero).astype(np.int8)
             elif format == 'I' and bzero == - np.iinfo(np.int16).min:
@@ -183,8 +190,14 @@ def read(self, filename, hdu=None, verbose=True):
             not key in standard_keys:
             self.add_keyword(key, header[key])
 
-    for comment in header.get_comment():
-        self.add_comment(comment)
+    try:
+        for comment in header.get_comment():
+            if isinstance(comment, pyfits.Card):
+                self.add_comment(comment.value)
+            else:
+                self.add_comment(comment)
+    except KeyError:
+        pass
 
     if hdu.name:
         self.table_name = str(hdu.name)
@@ -255,11 +268,16 @@ def _to_hdu(self):
     hdu.name = self.table_name
 
     for key in self.keywords:
+
         if len(key) > 8:
             keyname = "hierarch " + key
         else:
             keyname = key
-        hdu.header.update(keyname, self.keywords[key])
+
+        try:  # PyFITS 3.x
+            hdu.header[keyname] = self.keywords[key]
+        except KeyError:  # PyFITS 2.x
+            hdu.header.update(keyname, self.keywords[key])
 
     for comment in self.comments:
         hdu.header.add_comment(comment)
@@ -296,7 +314,10 @@ def write(self, filename, overwrite=False):
         _to_hdu(self).writeto(filename, output_verify='silentfix')
 
 
-def read_set(self, filename, verbose=True):
+# PyFITS can handle compression, so no decompression detection
+@auto_download_to_file
+@auto_fileobj_to_file
+def read_set(self, filename, memmap=False, verbose=True):
     '''
     Read all tables from a FITS file
 
@@ -304,6 +325,11 @@ def read_set(self, filename, verbose=True):
 
         *filename*: [ string ]
             The FITS file to read the tables from
+
+    Optional Keyword Arguments:
+
+        *memmap*: [ bool ]
+            Whether PyFITS should use memory mapping
     '''
 
     _check_pyfits_installed()
@@ -317,14 +343,20 @@ def read_set(self, filename, verbose=True):
         if not key[:4] in ['TFOR', 'TDIS', 'TDIM', 'TTYP', 'TUNI'] and \
             not key in standard_keys:
             self.add_keyword(key, header[key])
-
-    for comment in header.get_comment():
-        self.add_comment(comment)
+    try:
+        for comment in header.get_comment():
+            if isinstance(comment, pyfits.Card):
+                self.add_comment(comment.value)
+            else:
+                self.add_comment(comment)
+    except KeyError:
+        pass
 
     # Read in tables one by one
+    from .basetable import Table
     for hdu in _list_tables(filename):
-        table = atpy.Table()
-        read(table, filename, hdu=hdu, verbose=verbose)
+        table = Table()
+        read(table, filename, hdu=hdu, memmap=memmap, verbose=verbose)
         self.append(table)
 
 
@@ -353,11 +385,16 @@ def write_set(self, filename, overwrite=False):
 
     primary = pyfits.PrimaryHDU()
     for key in self.keywords:
+
         if len(key) > 8:
             keyname = "hierarch " + key
         else:
             keyname = key
-        primary.header.update(keyname, self.keywords[key])
+
+        try:  # PyFITS 3.x
+            primary.header[keyname] = self.keywords[key]
+        except KeyError:  # PyFITS 2.x
+            primary.header.update(keyname, self.keywords[key])
 
     for comment in self.comments:
         primary.header.add_comment(comment)
