@@ -1,20 +1,20 @@
+from __future__ import print_function, division
+
 import os
 from distutils import version
 import numpy as np
 import warnings
-import re
 
-from exceptions import TableException
-
-import atpy
-
-from helpers import smart_dtype
+from .exceptions import TableException
+from .helpers import smart_dtype
+from .decorators import auto_download_to_file, auto_decompress_to_fileobj, auto_fileobj_to_file
 
 vo_minimum_version = version.LooseVersion('0.3')
 
 try:
     from vo.table import parse
-    from vo.tree import VOTableFile, Resource, Table, Field
+    from vo.tree import VOTableFile, Resource, Field, Param
+    from vo.tree import Table as VOTable
     vo_installed = True
 except:
     vo_installed = False
@@ -49,6 +49,11 @@ def _list_tables(filename, pedantic=False):
     return tables
 
 
+# VO can handle file objects, but because we need to read it twice we don't
+# use that capability
+@auto_download_to_file
+@auto_decompress_to_fileobj
+@auto_fileobj_to_file
 def read(self, filename, pedantic=False, tid=-1, verbose=True):
     '''
     Read a table from a VOT file
@@ -95,13 +100,7 @@ def read(self, filename, pedantic=False, tid=-1, verbose=True):
 
     for field in table.fields:
 
-        if type(field.name) == str:
-            colname = field.name
-        else:
-            if type(field._ID) == str:
-                colname = field._ID
-            else:
-                raise Exception("Error reading in the VO table: no name or ID for field")
+        colname = field.ID
 
         data = table.array[colname]
 
@@ -111,18 +110,31 @@ def read(self, filename, pedantic=False, tid=-1, verbose=True):
 
         if self._masked:
             self.add_column(colname, data, \
-                unit=field.unit, mask=table.mask[colname])
+                unit=field.unit, mask=table.mask[colname], \
+                description=field.description)
         else:
             self.add_column(colname, data, \
-                unit=field.unit)
+                unit=field.unit, description=field.description)
+
+    for param in table.params:
+        self.add_keyword(param.ID, param.value)
 
 
-def _to_table(self, VOTable):
+def _to_table(self, vo_table):
     '''
     Return the current table as a VOT object
     '''
 
-    table = Table(VOTable)
+    table = VOTable(vo_table)
+
+    # Add keywords
+    for key in self.keywords:
+        if isinstance(self.keywords[key], basestring):
+            arraysize = '*'
+        else:
+            arraysize = None
+        param = Param(table, name=key, ID=key, value=self.keywords[key], arraysize=arraysize)
+        table.params.append(param)
 
     # Define some fields
 
@@ -133,6 +145,7 @@ def _to_table(self, VOTable):
 
         data = self.data[name]
         unit = self.columns[name].unit
+        description = self.columns[name].description
         dtype = self.columns[name].dtype
         column_type = smart_dtype(dtype)
 
@@ -164,15 +177,25 @@ def _to_table(self, VOTable):
         else:
             precision = None
 
-        fields.append(Field(VOTable, ID="col" + str(i), name=name, \
+        if datatype == 'char':
+            if arraysize is None:
+                arraysize = '*'
+            else:
+                raise ValueError("Cannot write vector string columns to VO files")
+
+        field = Field(vo_table, ID=name, name=name, \
                 datatype=datatype, unit=unit, arraysize=arraysize, \
-                precision=precision))
+                precision=precision)
+
+        field.description = description
+
+        fields.append(field)
 
     table.fields.extend(fields)
 
     table.create_arrays(n_rows)
 
-    # Character columns are stored as object columns in the VOTable
+    # Character columns are stored as object columns in the vo_table
     # instance. Leaving the type as string should work, but causes
     # a segmentation fault on MacOS X with Python 2.6 64-bit so
     # we force the conversion to object type columns.
@@ -229,19 +252,24 @@ def write(self, filename, votype='ascii', overwrite=False):
         else:
             raise Exception("File exists: %s" % filename)
 
-    VOTable = VOTableFile()
+    vo_table = VOTableFile()
     resource = Resource()
-    VOTable.resources.append(resource)
+    vo_table.resources.append(resource)
 
-    resource.tables.append(_to_table(self, VOTable))
+    resource.tables.append(_to_table(self, vo_table))
 
     if votype is 'binary':
-        VOTable.get_first_table().format = 'binary'
-        VOTable.set_all_tables_format('binary')
+        vo_table.get_first_table().format = 'binary'
+        vo_table.set_all_tables_format('binary')
 
-    VOTable.to_xml(filename)
+    vo_table.to_xml(filename)
 
 
+# VO can handle file objects, but because we need to read it twice we don't
+# use that capability
+@auto_download_to_file
+@auto_decompress_to_fileobj
+@auto_fileobj_to_file
 def read_set(self, filename, pedantic=False, verbose=True):
     '''
     Read all tables from a VOT file
@@ -262,8 +290,9 @@ def read_set(self, filename, pedantic=False, verbose=True):
 
     self.reset()
 
+    from .basetable import Table
     for tid in _list_tables(filename, pedantic=pedantic):
-        t = atpy.Table()
+        t = Table()
         read(t, filename, tid=tid, verbose=verbose, pedantic=pedantic)
         self.append(t)
 
@@ -291,15 +320,15 @@ def write_set(self, filename, votype='ascii', overwrite=False):
         else:
             raise Exception("File exists: %s" % filename)
 
-    VOTable = VOTableFile()
+    vo_table = VOTableFile()
     resource = Resource()
-    VOTable.resources.append(resource)
+    vo_table.resources.append(resource)
 
     for table_key in self.tables:
-        resource.tables.append(_to_table(self.tables[table_key], VOTable))
+        resource.tables.append(_to_table(self.tables[table_key], vo_table))
 
     if votype is 'binary':
-        VOTable.get_first_table().format = 'binary'
-        VOTable.set_all_tables_format('binary')
+        vo_table.get_first_table().format = 'binary'
+        vo_table.set_all_tables_format('binary')
 
-    VOTable.to_xml(filename)
+    vo_table.to_xml(filename)
